import discord, asyncio
from discord.ext import commands
from discord import Embed
from typing import List, Optional, Union


async def clear(msg):
    """helper function to remove reactions excepting forbidden
    either by context being a dm_channel or bot lacking perms"""

    if msg.guild:
        try:
            await msg.clear_reactions()
        except discord.errors.Forbidden:
            pass


async def arrows(ctx: commands.Context,
                 embed_list: List[Embed],
                 head: int = 0,
                 timeout: int = 30,
                 channel: Optional[discord.abc.Messageable] = None):
    """Sends multiple embeds with a reaction navigation menu.
    Intended to be used with discord.ext.commands

    :Parameters:
        ctx               - the context where this function is called.
        embed_list        - an ordered list containing the embeds to be sent.
        head              - the index in embed_list of the first Embed to be displayed.
        timeout (seconds) - the time before the bot closes the menu. This is reset with each interaction.
        channel           - the channel to be used for displaying the menu, defaults to ctx.channel.

    Returns:
        None
    """
    
    if not channel:
        channel = ctx.channel
    if len(embed_list) == 1:
        await ctx.send(embed=embed_list[0])
        return
    emojis = ['⏮', '◀', '▶', '⏭', '❌', '⏸']
    msg = await channel.send(embed=embed_list[head])
    for emoji in emojis:
        await msg.add_reaction(emoji)

    def check(payload):
        return payload.user_id == ctx.author.id and payload.emoji.name in emojis and msg.id == payload.message_id

    def return_head(head, emoji):
        if emoji == '⏮':  # return to the first Embed
            return 0
        elif emoji == '◀':  # one left
            return head - 1 if head > 0 else 0
        elif emoji == '▶':  # one right
            return head + 1 if head < (len(embed_list) - 1) else len(embed_list) - 1
        elif emoji == '⏭':  # go to the end
            return len(embed_list) - 1
        elif emoji == '❌':  # close the menu
            return False
        elif emoji == '⏸':  # remove the reactions and keep the selected embed open.
            return None

    head = head

    while True:
        try:
            payload = await ctx.bot.wait_for('raw_reaction_add', timeout=timeout, check=check)
        except asyncio.TimeoutError:
            try:  # This is done so that if the menu is called in a dm_channel context it omits deleting the reactions.
                await msg.clear_reactions()
            except discord.errors.Forbidden:
                pass
            break
        else:
            head = return_head(head, payload.emoji.name)

            if head is False:
                await clear(msg)
                await msg.edit(embed=discord.Embed(description='Closed by user.'))

            elif isinstance(head, int):
                try:
                    await msg.remove_reaction(payload.emoji, ctx.author)
                except discord.errors.Forbidden:
                    pass

                emb = embed_list[head]
                await msg.edit(embed=emb)

            elif head is None:
                await clear(msg)
                break


async def confirm(ctx: commands.Context,
                  msg: discord.Message,
                  lock: Union[discord.Member, discord.Role, bool, None] = True,
                  timeout: int = 30) -> Optional[bool]:
    """Helps to create a reaction menu to confirm an action.
    :Parameters:
        ctx                    - the context for the menu
        msg                    - the message to confirm or deny by the user.
        lock                   -
            :Bool:             -
                True (default) - the menu will only listen for the author's reactions.
                False          - ANY user can react to the menu
            :discord.Member:   - only target member will be able to react
            :discord.Role:     - ANY user with target role will be able to react.
        timeout (seconds)      - Timeout before the menu closes.

    Returns:
        True if the message was confirmed
        False if it was denied
        None if timeout
    """

    emojis = ['👍', '❌']
    for emoji in emojis:
        await msg.add_reaction(emoji)

    def check(payload):
        _checks = [
            payload.user_id != ctx.bot.user.id,
            payload.emoji.name in emojis,
            payload.message_id == msg.id,
        ]
        if lock:
            if isinstance(lock, bool):
                _checks.append(payload.user_id == ctx.author.id)
            elif isinstance(lock, discord.Member):
                _checks.append(payload.user_id == lock.id)
            elif isinstance(lock, discord.Role):
                _checks.append(lock in ctx.guild.get_member(payload.user_id).roles)
        return all(_checks)

    try:
        payload = await ctx.bot.wait_for('raw_reaction_add', check=check, timeout=timeout)
    except asyncio.TimeoutError:
        await clear(msg)
        return None
    else:
        await clear(msg)
        if payload.emoji.name == '👍':
            return True
        else:
            return False



from discord.ext import commands
from discord.ext.commands import ExtensionError
from discord import Embed, Color
from typing import Optional
import os


class UnknownCog(commands.CommandError):
    pass


class InvalidOption(commands.CommandError):
    pass


class OwnerCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    async def cog_check(self, ctx):
        if not await self.bot.is_owner(ctx.author):
            raise commands.NotOwner("This command can only be executed by the owner of the bot")
        else:
            return True

    @commands.command(hidden=True)
    async def cogs(
            self,
            ctx: commands.Context,
            option: Optional[str],
            cog: Optional[str],
            cogs_dir: str = "./cogs"):
        """
        Command to load, unload and reload extensions.
        Args:
            ctx: commands.Context
            option:
                load: loads :cog:
                unload: unloads :cog:
                reload: relaods :cog:
                list: lists available cogs in :cogs_dir:
            cog: filename of the extension to load, without the file extension ('my_cog' not 'my_cog.py')
            cogs_dir: directory to operate command, defaults to ./cogs

        Raises:
            InvalidOption if option is not recognized
            UnknownCog if cog name is not found within the specified directory
        """
        if option.lower() not in ['load', 'unload', 'reload', 'list']:
            raise InvalidOption(f"{option} is not valid. Valid options are:'load', 'unload', 'reload' and 'list' ")

        bot = ctx.bot

        actions = {
            'load': bot.load_extension,
            'unload': bot.unload_extension,
            'reload': bot.reload_extension
        }

        cogs = [filename[:-3] for filename in os.listdir(cogs_dir) if filename.endswith('.py')]

        if option.lower() == 'list':
            await ctx.send(embed=Embed(
                title="Available cogs:",
                description="".join(f"{i + 1}) {cog}\n" for i, cog in enumerate(cogs)),
                color=Color.blue()
            ))
        else:
            action = actions[option]

            if cog.lower() == 'all':
                msg = await ctx.send(f'Doing: {action.__name__} on all')
                for cog in cogs:
                    try:
                        action(f"{cogs_dir[2:]}.{cog}")
                    except ExtensionError:
                        pass
            elif cog not in cogs:
                raise UnknownCog(f"{cog} is not a valid cog, check spelling and verify directory.")
            else:
                msg = await ctx.send(f'Doing: {action.__name__} on {cog}')
                action(f"{cogs_dir[2:]}.{cog}")

            await msg.edit(content=f'**Done!**')



from datetime import timedelta
from typing import Dict
from collections import ChainMap
import re

class InvalidTimeString(Exception):
    pass

def parse_time(string: str) -> timedelta:
    """
    Converts a string with format <number>[s|m|h|d] to a timedelta object
    <number> must be convertible to float.
    Uses regex to match groups and consumes all groups into one timedelta.

    Units:
        s: second
        m: minute
        h: hour
        d: day

    Args:
        string: with format <number>[s|m|h|d].
            parse_time("2h") == timedelta(hours=2)

    Returns:
        timedelta

    Raises:
        ValueError: if string number cannot be converted to float
        InvalidTimeString: if string isn't in the valid form.

    """


    def parse(time_string: str) -> Dict:
        units = {'d': 'days', 's': 'seconds', 'm': 'minutes', 'h': 'hours', 'w': 'weeks'}

        unit = time_string[-1]

        amount = float(time_string[:-1])
        return {units[unit]: amount}

    pattern = r"(\d+[.]?\d?[s|m|h|d]{1})\s?"
    matched = re.findall(pattern, string)

    if not matched:
        raise InvalidTimeString("Invalid string format. Time must be in the form <number>[s|m|h|d].")

    time_dict = dict(ChainMap(*[parse(d) for d in matched]))

    return timedelta(**time_dict)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dpytools']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'dpytools',
    'version': '0.0.7a3',
    'description': 'Simple tools to build discord bots using discord.py',
    'long_description': '# dpytools\nToolset to speed up developing discord bots using discord.py\n\n<hr>\n\n## Status of the project\n\nEarly development. As such its expected to be unstable and unsuited for production.\n\n## Components\n\n**Module** | **Functions** | **Notes**\n--- | --- | --- \nmenus | arrows  | Dispays a menu made from passed Embeds with navigation by reaction.\n.| confirm | Returns the user reaction to confirm or deny a passed message.\nembeds | paginate_to_embeds | Paginates a long text into a list of embeds.\nparsers| parse_time | Parses strings with the format "2h15m" to a timedelta object.\nowner_cog | - | collection of useful commands for the bot owner.\nMore to come... \n\n<hr>\n\n# Contributing\nFeel free to make a pull request.',
    'author': 'chrisdewa',
    'author_email': 'alexdewa@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/chrisdewa/dpytools',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from discord.ext import commands

class IncorrectGuild(commands.CommandError):
    pass


def only_this_guild(guild_id: int):
    """This checks returns true if ctx.guild has same id as param :guild_id:

    Raises:
        commands.NoPrivateMessage if ran from DM
        Custom IncorrectGuild commands.CommandError if id doesn't check

    Returns:
        True if guild_id == ctx.guild.id
    """
    async def predicate(ctx):
        if ctx.guild is None:
            raise commands.NoPrivateMessage()
        elif guild_id != ctx.guild.id:
            raise IncorrectGuild()
        return True

    return commands.check(predicate)